#!/bin/sh
###########################################################
#
# asu.sh
#
# abfackeln's server utilities
# for unreal tournament
#
# Copyright (C) 2001 abfackeln@abfackeln.com
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 
# Usage:
# sh asu.sh
#
###########################################################

Read_Vars()
{
   MatchIni IpDrv.TcpNetDriver DownloadManagers IpDrv.HTTPDownload $UT_MAIN_INI
   if [ -n "$INI_VAL" ]; then
      MSG_REDIRECT="Disable Download Redirection"
      OPT_REDIRECT=1
   else
      MSG_REDIRECT="Enable Download Redirection"
      OPT_REDIRECT=0
   fi

   ReadIni Engine.GameInfo bBatchLocal $UT_MAIN_INI
   if [ "$INI_VAL" = "True" ]; then
      MSG_NGLS="Disable NG Local Stats"
      OPT_NGLS=1
   else
      MSG_NGLS="Enable NG Local Stats (requires JDK 1.2)"
      OPT_NGLS=0
   fi

   if [ -f "$UT_USER_INI.old-bots" ]; then
      MSG_BINBOTS="Restore Old Bot Settings"
      OPT_BINBOTS=1
   else
      MSG_BINBOTS="Use Binary Bot Settings"
      OPT_BINBOTS=0
   fi

   if [ -d "$UT_DIR/Web/graphic" ]; then
      MSG_WEBTEXT="Restore Graphic Web Admin"
      OPT_WEBTEXT=1
   else
      MSG_WEBTEXT="Set Text-only Web Admin"
      OPT_WEBTEXT=0
   fi
}

Read_Optim_Vars()
{
   Read_Optim_Std C
   Read_Optim_Std I
}

Read_Optim_Std ()
{
   eval ARG="\$ARG_OPTIM_$1"
   eval DEF="\$DEF_OPTIM_$1"
   eval PRF="\$PRF_OPTIM_$1"

   ReadIni $ARG $UT_MAIN_INI
   CUR=$INI_VAL
   eval MSG_OPTIM_$1=\"\(Def: \$DEF, Opt: \$PRF, Cur: \$CUR\)\"
}

Optimize_Std ()
{
   eval ARG="\$ARG_OPTIM_$1"
   eval PRF="\$PRF_OPTIM_$1"

   echo -n "New value for $ARG (Suggested value $PRF) : "
   read SET_TO

   if [ -z "$SET_TO" ]; then
      SET_TO="$PRF"
   fi

   MESSAGE="$ARG set to $SET_TO"
   SetIni $ARG "$SET_TO" $UT_MAIN_INI
}


### ini file modifying scripts


AddIni()
{
   TMP="/tmp/ut.ini.$$.tmp"
   INI="$4"

   awk 'BEGIN { eval ARGV[1]; eval ARGV[2]; eval ARGV[3]; instate = 0; done = 0; }
/^\[/ { if ($1 ~ "\\[" zone "\\]") { instate = 1; } else { instate = 0; } }
{
   if (instate == 2) {
      if (length($1) <= 1) {
         print item "=" value; done = 1;
      } else if ($1 ~ "^" item "=" value "[ \t\f\n\r\v]*$") {
         instate = 0; done = 1;
      }
   } else if (instate == 1) {
      instate = 2;
   }
   print $0;
}
END { if (done == 0) { print "[" zone "]"; print item "=" value; print; } }
' "zone=$1" "item=$2" "value=$3" < "$INI" > "$TMP"

   if [ $? -eq 0 ]; then
      mv "$TMP" "$INI"
   fi
}

UnAddIni()
{
   TMP="/tmp/ut.ini.$$.tmp"
   INI="$4"

   awk 'BEGIN { eval ARGV[1]; eval ARGV[2]; eval ARGV[3]; instate = 0; }
/^\[/ { if ($1 ~ "\\[" zone "\\]") { instate = 1; } else { instate = 0; } }
{
   if (instate == 2) {
      if (length($1) <= 1) {
         print $0;
      } else if ($1 ~ "^" item "=" value "[ \t\f\n\r\v]*$") {
      } else {
         print $0;
      }
   } else if (instate == 1) {
      print $0; instate = 2;
   } else {
      print $0;
   }
}' "zone=$1" "item=$2" "value=$3" < "$INI" > "$TMP"

   if [ $? -eq 0 ]; then
      mv "$TMP" "$INI"
   fi
}

SetIni()
{
   TMP="/tmp/ut.ini.$$.tmp"
   INI="$4"

   awk 'BEGIN { eval ARGV[1]; eval ARGV[2]; eval ARGV[3]; instate = 0; done = 0; }
/^\[/ { if ($1 ~ "\\[" zone "\\]") { instate = 1; } else { instate = 0; } }
{
   if (instate == 2) {
      if (length($1) <= 1) {
         print item "=" value; print $0; done = 1;
      } else if ($1 ~ "^" item "=") {
         print item "=" value; instate = 0; done = 1;
      } else {
         print $0;
      }
   } else if (instate == 1) {
      print $0; instate = 2;
   } else {
      print $0;
   }
}
END { if (done == 0) { print "[" zone "]"; print item "=" value; print; } }
' "zone=$1" "item=$2" "value=$3" < "$INI" > "$TMP"

   if [ $? -eq 0 ]; then
      mv "$TMP" "$INI"
   fi
}

UnSetIni()
{
   TMP="/tmp/ut.ini.$$.tmp"
   INI="$3"

   awk 'BEGIN { eval ARGV[1]; eval ARGV[2]; eval ARGV[3]; instate = 0; }
/^\[/ { if ($1 ~ "\\[" zone "\\]") { instate = 1; } else { instate = 0; } }
{
   if (instate == 2) {
      if (length($1) <= 1) {
         print $0;
      } else if ($1 ~ "^" item "=") {
      } else {
         print $0;
      }
   } else if (instate == 1) {
      print $0; instate = 2;
   } else {
      print $0;
   }
}' "zone=$1" "item=$2" < "$INI" > "$TMP"

   if [ $? -eq 0 ]; then
      mv "$TMP" "$INI"
   fi
}

MatchIni()
{
   TMP="/tmp/ut.ini.$$.tmp"
   INI="$4"

   awk 'BEGIN { eval ARGV[1]; eval ARGV[2]; eval ARGV[3]; instate = 0; }
/^\[/ { if ($1 ~ "\\[" zone "\\]") { instate = 1; } else { instate = 0; } }
{
   if (instate == 2) {
      if (length($1) <= 1) {
      } else if ($1 ~ "^" item "=" value "[ \t\f\n\r\v]*$") {
         print "match";
      }
   } else if (instate == 1) {
      instate = 2;
   }
}' "zone=$1" "item=$2" "value=$3" < "$INI" > "$TMP"

   INI_VAL=`cat $TMP`
   rm -f $TMP
}

ReadIni()
{
   TMP="/tmp/ut.ini.$$.tmp"
   INI="$3"

   awk 'BEGIN { eval ARGV[1]; eval ARGV[2]; eval ARGV[3]; instate = 0; FS="="; }
/^\[/ { if ($1 ~ "\\[" zone "\\]") { instate = 1; } else { instate = 0; } }
{ if ((instate == 1) && ($1 == item)) { gsub(/[ \t\f\n\r\v]/, "", $2); print $2; } }
' "zone=$1" "item=$2" < "$INI" > "$TMP"

   if [ $? -eq 0 ]; then
      INI_VAL=`head -1 $TMP`
   else
      INI_VAL=""
   fi
   rm -f $TMP
}

Install_Package ()
{
   if [ -f "$ABF_DIR/$1-patch.tar.gz" ]; then
      cd $UT_DIR
      tar zxpf "$ABF_DIR/$1-patch.tar.gz"
      cd $ABF_DIR/..
      if [ -f "$UT_DIR/$1-install.asu" ]; then
         . "$UT_DIR/$1-install.asu"
         rm -f "$UT_DIR/$1-install.asu"
      fi
      Add_Message "+++ $1 patch is now installed!"
   else
      Add_Message "!!! Error: can not find patch file: $ABF_DIR/$1-patch.tar.gz"
   fi
}

Guess_Paths ()
{
   GetPath
   ABF_DIR=$RETVAL
   UT_DIR=`echo "$ABF_DIR" | sed s/[/]Abfackeln.com//`
   SYS_DIR="$UT_DIR/System"
   ABF_DIR="$ABF_DIR/Lib"

   UT_USER_INI="$SYS_DIR/User.ini"
   UT_MAIN_INI="$SYS_DIR/UnrealTournament.ini"
}

Check_Paths ()
{
   if [ ! -f "$UT_MAIN_INI" ]; then
      echo
      echo "Can not find '$UT_MAIN_INI'"
      echo "Check that your directory name and file names are correct."
      echo
      Menu_Main_D
   else
      if [ ! -f "$UT_USER_INI" ]; then
         echo
         echo "Can not find '$UT_USER_INI'"
         echo "Check that your directory name and file names are correct."
         echo
         Menu_Main_D
      fi
   fi
}


### this is the "main loop"


Initialization()
{
   VERSION="0.1"

   ### basic init routines

   if [ "$0" = "-bash" ];then
      echo "usage:"
      echo "sh asu.sh"
      echo "sh asu.sh <the ut directory location>"
      exit -2
   fi

   Guess_Paths
   Check_Paths

   ### the next two lines create the $NL newline var
   NL="
"
   ### do not change the spacing or anything in the above two lines


   ### Optimization settings:

   ARG_OPTIM_C="IpDrv.TcpNetDriver MaxClientRate"
   DEF_OPTIM_C="20000"
   PRF_OPTIM_C="5000"

   ARG_OPTIM_I="IpDrv.TcpNetDriver InitialConnectTimeout"
   DEF_OPTIM_I="500.0"
   PRF_OPTIM_I="35.0"

   Menu_Main
}

GetPath()
{
   if [ -z `echo "$0" | grep "^[/]"` ];then
      RETVAL=`pwd`
      RETVAL="$RETVAL/$0"
   else
      RETVAL=$0
   fi
   RETVAL=`echo "$RETVAL" | sed s/[/]asu.sh//`
}

TopDisplay()
{
   echo "asu version $VERSION, Copyright (C) 2001 abfackeln@abfackeln.com"
   echo "asu is free software and comes with ABSOLUTELY NO WARRANTY."
   echo "Enter H for help and more copyright information."
   echo
}

Add_Message ()
{
   MESSAGE="$MESSAGE""$NL  $1"
}

Read_Input()
{
   echo
   if [ -n "$MESSAGE" ]; then
      echo "> $MESSAGE"
      echo
      MESSAGE=""
   fi
   echo "Unreal Tournament Location: $UT_DIR"
   echo "Enter a letter from the above menu."
   echo -n "? "
   read MENU_SEL

   ### get first character of MENU_SEL
   MENU_SEL=`echo "$MENU_SEL" | sed "s/\(.\).*/\1/g"`
}

Generic_Bots ()
{
   BOT_FACE=SGirlSkins.Cathode
   BOT_SKIN=SGirlSkins.fwar
   BOT_CLASS=BotPack.TFemale2Bot
   BOT_VOICE=BotPack.VoiceBoss

   MYFILE=/tmp/sed.tmp.$$
   TMPFILE=/tmp/sed.tmpb.$$

   cat $UT_USER_INI > $MYFILE

   VAR=0
   MAX=32

   while [ $VAR -le $MAX ]; do

      REM=$VAR
      BIN=
      SIG=2

      while [ $SIG -le $MAX ]; do

         if [ $[$REM % $SIG] -gt 0 ]; then
            BIN=1$BIN
            REM=$[ $REM - $[SIG / 2] ]
         else
            BIN=0$BIN
         fi

         SIG=$[ $SIG * 2 ]

      done

      sed s/^BotNames\\[$VAR\\].*/BotNames[$VAR]=$BIN/ $MYFILE > $TMPFILE
      mv $TMPFILE $MYFILE

      VAR=$[ $VAR + 1 ]

   done

   sed "s/^BotFaces\[\([0-9]*\)\].*/BotFaces[\1]=$BOT_FACE/" $MYFILE > $TMPFILE
   sed "s/^BotClasses\[\([0-9]*\)\].*/BotClasses[\1]=$BOT_CLASS/" $TMPFILE > $MYFILE
   sed "s/^BotSkins\[\([0-9]*\)\].*/BotSkins[\1]=$BOT_SKIN/" $MYFILE > $TMPFILE

   ID=0
   while [ $ID -lt 32 ]; do
           echo "VoiceType[$ID]=$BOT_VOICE" >> $TMPFILE
      ID=$(( $ID + 1 ))
   done

   cat $TMPFILE > $UT_USER_INI

   rm -f $MYFILE $TMPFILE
}

### menu functions ###

Menu_Main ()
{
   MESSAGE=""
   while [ "$MENU_SEL" != "x" ]; do
      clear
      Read_Vars
      TopDisplay

      echo "MAIN MENU"
      echo "---------"
      echo
      echo "B) Set Minimum Number of Bots"
      echo "D) Specify UT Directory and active .ini files"
      echo "G) $MSG_BINBOTS"
      echo "H) Help and Copyright Information"
      echo "I) Install Init Script"
      echo "M) Mods and Patches Menu"
      echo "N) $MSG_NGLS"
      echo "O) Optimizations Menu"
      echo "P) Set Passwords"
      echo "R) $MSG_REDIRECT"
      echo "S) Server Info and MOTD"
      echo "W) $MSG_WEBTEXT"
      echo "X) Exit"
      Read_Input

      ### now process the menu selection

      case "$MENU_SEL" in
      b|B)
         Menu_Main_B
         ;;
      d|D)
         Menu_Main_D
         ;;
      g|G)
         Menu_Main_G
         ;;
      h|H)
         Menu_Main_H
         ;;
      i|I)
         Menu_Main_I
         ;;
      m|M)
         Menu_Main_M
         ;;
      n|N)
         Menu_Main_N
         ;;
      o|O)
         Menu_Main_O
         ;;
      p|P)
         Menu_Main_P
         ;;
      r|R)
         Menu_Main_R
         ;;
      s|S)
         Menu_Main_S
         ;;
      w|W)
         Menu_Main_W
         ;;
      X)
         MENU_SEL=x
         ;;
      *)
         MESSAGE="Unknown Command."
         ;;
      esac
   done
}

###  B is to set the number of bots

Menu_Main_B ()
{
   echo -n "Enter the minimum number of bots you want on your server: "
   read READ_VAL
   SetIni Botpack.DeathMatchPlus MinPlayers "$READ_VAL" $UT_MAIN_INI
   SetIni Botpack.CTFGame MinPlayers "$READ_VAL" $UT_MAIN_INI
   SetIni Botpack.Domination MinPlayers "$READ_VAL" $UT_MAIN_INI
   SetIni Botpack.Assault MinPlayers "$READ_VAL" $UT_MAIN_INI
   SetIni Botpack.TeamGamePlus MinPlayers "$READ_VAL" $UT_MAIN_INI
   MESSAGE="MinPlayers: $READ_VAL"
}

###  D is to set the directories

Menu_Main_D ()
{
   echo
   echo -n "Enter the full path of the UT game directory: "
   read UT_DIR
   SYS_DIR="$UT_DIR/System"
   if [ ! -d "$SYS_DIR" ]; then
      echo "The $SYS_DIR directory does not exist."
      Menu_Main_D
   else
      echo
      DEF=UnrealTournament.ini
      echo -n "Name of the main .ini file [$DEF]: "
      read READ_VAL
      if [ -z "$READ_VAL" ]; then
         READ_VAL="$DEF"
      fi
      UT_MAIN_INI="$SYS_DIR/$READ_VAL"
      echo
      DEF=User.ini
      echo -n "Name of the main .ini file [$DEF]: "
      read READ_VAL
      if [ -z "$READ_VAL" ]; then
         READ_VAL="$DEF"
      fi
      UT_USER_INI="$SYS_DIR/$READ_VAL"
      MESSAGE="Unreal Tournament game directory: $UT_DIR"
      Add_Message "Main .ini file: $UT_MAIN_INI"
      Add_Message "User .ini file: $UT_USER_INI"
      Check_Paths
   fi
}

###  G is the generic bots

Menu_Main_G ()
{
      TMP="$UT_USER_INI.old-bots"
      if [ $OPT_BINBOTS -ne 0 ]; then
         mv -f "$TMP" "$UT_USER_INI"
         MESSAGE="Restored Old Bot Settings."
      else
         cp -f "$UT_USER_INI" "$TMP"
         echo "Generating Binary Bots ..."
         Generic_Bots
         MESSAGE="Created Binary Bot Settings."
      fi
}

###  H is help

Menu_Main_H ()
{
   clear
   TopDisplay
   zcat Lib/help.txt.gz | more
   echo
   echo -n "Press enter to continue or enter L for legal and copyright information: "
   read READ_VAL
   READ_VAL=`echo "$READ_VAL" | sed "s/\(.\).*/\1/g"`
   if [ "$READ_VAL" = "L" ]; then
      READ_VAL="l"
   fi
   if [ "$READ_VAL" = "l" ]; then
      zcat Lib/legal.txt.gz | more
      echo
      echo -n "Press enter to continue or enter C for copyright information: "
      read READ_VAL
      READ_VAL=`echo "$READ_VAL" | sed "s/\(.\).*/\1/g"`
      if [ "$READ_VAL" = "C" ]; then
         READ_VAL="c"
      fi
      if [ "$READ_VAL" = "c" ]; then
         zcat Lib/gpl.txt.gz | more
         echo -n "Press enter to continue: "
         read READ_VAL
      fi
   fi
}

###  I is the init script

Menu_Main_I ()
{
   echo
   echo "if you intend to run this program as a user other than root, then enter"
   echo "the username here .. if you run as root then it will suid to this user"
   echo

   ## read user id

   USERID=
   while [ -z "$USERID" ]; do
      echo -n "Enter the userid that the ucc program should run as [nobody]: "
      read USERID
      if [ -z "$USERID" ]; then
         USERID="nobody"
      fi
      if [ "$USERID" = "root" ]; then
         echo "invalid user id; should not run as root, pick someone else"
         USERID=
      fi
      XXTMP=`grep ^$USERID: /etc/passwd`
      if [ "$XXTMP" = "" ]; then
         echo "invalid user id"
         USERID=
      fi
      XXTMP=
   done

   ## read game port

   MYPORT=
   while [ -z "$MYPORT" ]; do
      echo -n "Enter the port where the ucc program should run [7777]: "
      read MYPORT
      MYPORT=$(($MYPORT + 0))
      if [ $MYPORT -eq 0 ]; then
         MYPORT="7777"
      fi
   done

   ### read game type

   MYGAME=""
   while [ -z "$MYGAME" ]; do
      echo -n "Enter the game type 1=DM, 2=CTF, 3=DOM, 4=AS [1]: "
      read GAMETYPE
      if [ -z "$GAMETYPE" ]; then
         GAMETYPE="1"
      fi
      if [ "$GAMETYPE" = "1" ]; then
         GAMETYPE="DM"
         MYGAME="DeathMatchPlus"
      else
         if [ "$GAMETYPE" = "2" ]; then
            GAMETYPE="CTF"
            MYGAME="CTFGame"
         else
            if [ "$GAMETYPE" = "3" ]; then
               GAMETYPE="DOM"
               MYGAME="Domination"
            else
               if [ "$GAMETYPE" = "4" ]; then
                  GAMETYPE="AS"
                  MYGAME="Assault"
               fi
            fi
         fi
      fi
   done

   ## find a map based on game type

   echo
   ls $UT_DIR/Maps/$GAMETYPE-*.unr | sed -e "s/^.*Maps\/$GAMETYPE-\(.*\)\.unr/\1/g" | awk '{ printf ("%s, ", $0); }'
   DEF=`ls $UT_DIR/Maps/$GAMETYPE-*.unr | sed -e "s/^.*Maps\/$GAMETYPE-\(.*\)\.unr/\1/g" | awk '{ printf ("%s\n", $0); }' | head -1`
   echo
   while [ -z "$MAPNAME" ]; do
      echo
      echo -n "Select a map to start the game with [$DEF]: "
      read XXTMP
      if [ -z "$XXTMP" ]; then
         XXTMP=$DEF
      fi
      MAPNAME=`ls $UT_DIR/Maps/$GAMETYPE-*.unr | sed -e "s/^.*Maps\/$GAMETYPE-\(.*\)\.unr/\1/g" | awk 'BEGIN { eval ARGV[1]; } { if (tolower($0) == tolower(mygt)) { printf("%s", $0); } }' mygt=$XXTMP`
   done
   MYMAP="$GAMETYPE-$MAPNAME"

   ## add mods

   MYMODS=

   ## allow multihome

   MYIP=

   ###

   MYNAME="ucc-$MYPORT"
   TMPA="/tmp/uccinita.$$"
   TMPB="/tmp/uccinitb.$$"
   TMPC="$UT_DIR/$MYNAME.init"

   cp -f $ABF_DIR/ucc.init $TMPA

   sed "s|^MYNAME=.*|MYNAME=$MYNAME|" $TMPA > $TMPB
   sed "s|^MYDIR=.*|MYDIR=$UT_DIR|" $TMPB > $TMPA
   sed "s|^MYGAME=.*|MYGAME=$MYGAME|" $TMPA > $TMPB
   sed "s|^MYMAP=.*|MYMAP=$MYMAP|" $TMPB > $TMPA
   sed "s|^MYPORT=.*|MYPORT=$MYPORT|" $TMPA > $TMPB
   sed "s|^MYMODS=.*|MYMODS=$MYMODS|" $TMPB > $TMPA
   sed "s|^MYUSERID=.*|MYUSERID=$USERID|" $TMPA > $TMPB
   sed "s|^MYIP=.*|MYIP=$MYIP|" $TMPB > $TMPA
   sed "s|^MYINI=.*|MYINI=$UT_MAIN_INI|" $TMPA > $TMPC

   rm -f $TMPA $TMPB
   chmod +x $TMPC

   MESSAGE="Created $TMPC"
}

###  M is the mod menu

Menu_Main_M ()
{
   MESSAGE=""
   while [ "$MENU_SEL" != "x" ]; do
      clear
      TopDisplay

      echo "MODS AND PATCHES MENU"
      echo "---------------------"
      echo
      echo "I) Install a Umod file"
      echo "L) List Currently Installed Umods"
      echo "P) Patches"
      echo "U) Uninstall Umod"
      echo "V) Verify ServerPackages"
      echo "X) Exit to Main Menu"
      Read_Input

      ### now process the menu selection

      case "$MENU_SEL" in
      I|i)
         Menu_Mod_I
         ;;
      L|l)
         Menu_Mod_L
         ;;
      P|p)
         Menu_Mod_P
         ;;
      U|u)
         Menu_Mod_U
         ;;
      V|v)
         Menu_Mod_V
         ;;
      X|x)
         MENU_SEL=x
         ;;
      *)
         MESSAGE="Unknown Command."
         ;;
      esac
   done
   MENU_SEL=
}

###  N is the ngLocalStats

Menu_Main_N ()
{
   if [ $OPT_NGLS -eq 0 ]; then
      SetIni Engine.GameInfo bBatchLocal True $UT_MAIN_INI
      MESSAGE="Enabling ngLocalStats (requires JDK 1.2)"
   else
      SetIni Engine.GameInfo bBatchLocal False $UT_MAIN_INI
      MESSAGE="Disabling ngLocalStats"
   fi
}

###  O is the optimization menu

Menu_Main_O ()
{
   MESSAGE=""
   while [ "$MENU_SEL" != "x" ]; do
      clear
      Read_Optim_Vars
      TopDisplay

      echo "OPTIMIZATION MENU"
      echo "-----------------"
      echo
      echo "C) $ARG_OPTIM_C $MSG_OPTIM_C"
      echo "I) $ARG_OPTIM_I $MSG_OPTIM_I"
      echo "X) Exit to Main Menu"
      Read_Input

      ### now process the menu selection

      case "$MENU_SEL" in
      c|C)
         Optimize_Std C
         ;;
      i|I)
         Optimize_Std I
         ;;
      x|X)
         MENU_SEL=x
         ;;
      *)
         MESSAGE="Unknown Command."
         ;;
      esac
   done
   MENU_SEL=
}

###  P is the redirect

Menu_Main_P ()
{
   echo -n "Enter port number for the Web Admin (0 to disable): "
   read READ_VAL
   if [ -z "$READ_VAL" ]; then
      READ_VAL=7776
   fi
   if [ "$READ_VAL" = "0" ]; then
      SetIni UWeb.WebServer bEnabled False $UT_MAIN_INI
      MESSAGE="Web admin disabled"
   else
      SetIni UWeb.WebServer ListenPort "$READ_VAL" $UT_MAIN_INI
      SetIni UWeb.WebServer bEnabled True $UT_MAIN_INI
      MESSAGE="Web admin enabled"
      Add_Message "UWeb.WebServer.ListenPort: $READ_VAL"

      echo -n "Enter a username to enter the Web Admin: "
      read READ_VAL
      SetIni UTServerAdmin.UTServerAdmin AdminUsername "$READ_VAL" $UT_MAIN_INI
      Add_Message "UTServerAdmin.UTServerAdmin.AdminUsername: $READ_VAL"
      echo -n "Enter a password to enter the Web Admin: "
      read READ_VAL
      SetIni UTServerAdmin.UTServerAdmin AdminPassword "$READ_VAL" $UT_MAIN_INI
      Add_Message "UTServerAdmin.UTServerAdmin.AdminPassword: $READ_VAL"
   fi

   echo -n "Server Console Admin password (blank to disable): "
   read READ_VAL
   if [ -z "$READ_VAL" ]; then
      SetIni Engine.GameInfo AdminPassword "" $UT_MAIN_INI
      Add_Message "Private Admin Password disabled"
   else
      SetIni Engine.GameInfo AdminPassword "$READ_VAL" $UT_MAIN_INI
      Add_Message "Engine.GameInfo.AdminPassword: $READ_VAL"
   fi

   echo -n "Private server game password (blank to disable): "
   read READ_VAL
   if [ -z "$READ_VAL" ]; then
      SetIni Engine.GameInfo GamePassword "" $UT_MAIN_INI
      Add_Message "Private Game Password disabled"
   else
      SetIni Engine.GameInfo GamePassword "$READ_VAL" $UT_MAIN_INI
      Add_Message "Engine.GameInfo.GamePassword: $READ_VAL"
   fi
}

###  R is the redirect

Menu_Main_R ()
{
   if [ $OPT_REDIRECT -eq 0 ]; then
      MESSAGE="Enabling Download Redirect"
      SetIni IpDrv.HTTPDownload RedirectToURL http://uz.unrealengine.com/utpackages_dm/ $UT_MAIN_INI
      SetIni IpDrv.HTTPDownload UseCompression True $UT_MAIN_INI
      SetIni IpDrv.HTTPDownload ProxyServerPort 3128 $UT_MAIN_INI
      SetIni IpDrv.HTTPDownload ProxyServerPort "" $UT_MAIN_INI
      SetIni IpDrv.TcpNetDriver AllowDownloads True $UT_MAIN_INI
      SetIni IpDrv.TcpNetDriver MaxDownloadSize 100000 $UT_MAIN_INI
      UnSetIni IpDrv.TcpNetDriver DownloadManagers $UT_MAIN_INI
      AddIni IpDrv.TcpNetDriver DownloadManagers IpDrv.HTTPDownload $UT_MAIN_INI
      AddIni IpDrv.TcpNetDriver DownloadManagers Engine.ChannelDownload $UT_MAIN_INI
   else
      MESSAGE="Disabling Download Redirect"
      UnAddIni IpDrv.TcpNetDriver DownloadManagers IpDrv.HTTPDownload $UT_MAIN_INI
   fi
}

###  S is the server settings

Menu_Main_S ()
{
   echo
   echo -n "Enter a name for your server: "
   read VAL
   MESSAGE="ServerName=$VAL"
   SetIni Engine.GameReplicationInfo ServerName "$VAL" $UT_MAIN_INI

   echo -n "Enter the name of the server administrator: "
   read VAL
   Add_Message "AdminName=$VAL"
   SetIni Engine.GameReplicationInfo AdminName "$VAL" $UT_MAIN_INI

   echo -n "Enter an administrator email address: "
   read VAL
   Add_Message "AdminEmail=$VAL"
   SetIni Engine.GameReplicationInfo AdminEmail "$VAL" $UT_MAIN_INI

   echo "Enter 4 lines for a MOTD:"
   echo -n "> "
   read VAL
   Add_Message "MOTDLine1=$VAL"
   SetIni Engine.GameReplicationInfo MOTDLine1 "$VAL" $UT_MAIN_INI
   echo -n "> "
   read VAL
   Add_Message "MOTDLine2=$VAL"
   SetIni Engine.GameReplicationInfo MOTDLine2 "$VAL" $UT_MAIN_INI
   echo -n "> "
   read VAL
   Add_Message "MOTDLine3=$VAL"
   SetIni Engine.GameReplicationInfo MOTDLine3 "$VAL" $UT_MAIN_INI
   echo -n "> "
   read VAL
   Add_Message "MOTDLine4=$VAL"
   SetIni Engine.GameReplicationInfo MOTDLine4 "$VAL" $UT_MAIN_INI
}

###  W is the web admin

Menu_Main_W ()
{
   TMP="$UT_DIR/xxxx"
   if [ $OPT_WEBTEXT -ne 0 ]; then
      mv "$UT_DIR/Web/" "$TMP/"
      mv "$TMP/graphic/" "$UT_DIR/Web/"
      mv "$TMP/" "$UT_DIR/Web/plaintext/"
      MESSAGE="Restored Graphical Web Admin Interface."
   else
      mv "$UT_DIR/Web/" "$TMP/"
      mv "$TMP/plaintext/" "$UT_DIR/Web/"
      mv "$TMP/" "$UT_DIR/Web/graphic/"
      MESSAGE="Using Text-only Web Admin Interface."
   fi
}

###  Mod Menu

Menu_Mod_P ()
{
   ABF_INI=$SYS_DIR/abfackeln.ini
   touch $ABF_INI
   MESSAGE="Summary of patches:"

   for TMP in `ls Lib/*-*-patch.tar.gz | sed -e "s/^Lib\/\(.*\)-\(.*\)-patch\.tar\.gz$/PACKAGE=\\1;VERS=\\2/"`; do
      eval $TMP
      ReadIni abfackeln.patches $PACKAGE $ABF_INI
      INST="$PACKAGE-$VERS-patch.tar.gz"
      if [ "$VERS" != "$INI_VAL" ]; then
         echo
         if [ -n "$INI_VAL" ]; then
            echo "You currently have $PACKAGE version $INI_VAL installed"
            echo "-- You will not be able to undo the following patch  --"
         fi
         echo -n "Do you want to patch $PACKAGE to version $VERS (y/n) ? "
         read RET_VAL
         if [ "$RET_VAL" = "y" ]; then
            RET_VAL=`echo $VERS | sed -e "s/-.*$//"`
            if [ -n "$RET_VAL" ]; then
               VERS="$RET_VAL"
            fi
            SetIni abfackeln.patches $PACKAGE $VERS $ABF_INI
            Install_Package "$PACKAGE-$VERS"
         else
            Add_Message "--- $PACKAGE $VERS patch NOT installed."
         fi
      else
         Add_Message "--- $PACKAGE $VERS was already installed."
      fi
   done
}

Menu_Mod_I ()
{
   echo -n "Enter the full name of the .umod file: "
   read READ_VAL
   MESSAGE="Unable to find file: $READ_VAL"
   if [ -f "$READ_VAL" ]; then
      SUFFIX=`echo "$READ_VAL" | sed -e "s/^.*[.]\([^.]*\)/\\1/"`
      if [ "$SUFFIX" = "zip" ]; then
         TMP=`echo "$READ_VAL" | sed -e "s/^\(.*[/]\)\?\([^.]*\)\.zip/\\2/" -e "y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/"`
         unzip -C -L -d /tmp "$READ_VAL"
         READ_VAL="/tmp/$TMP.umod"
         SUFFIX="umod"
      fi
      if [ "$SUFFIX" = "umod" ]; then
         Lib/umodpack-abf/umod -b $UT_DIR -i $READ_VAL
         Menu_Mod_L
      else
         MESSAGE="Failed:  Must be a .umod or .zip file"
      fi
   fi
}

Menu_Mod_L ()
{
   MESSAGE="Installed Umods:$NL"`Lib/umodpack-abf/umod -b $UT_DIR | awk '{ print "  " $0; }'`
}

Menu_Mod_U ()
{
   echo -n "Enter the name of the umod package: "
   read READ_VAL
   MESSAGE="Unable to find file: $READ_VAL"
   if [ -n "$READ_VAL" ]; then
      Lib/umodpack-abf/umod -b $UT_DIR -u $READ_VAL
      Menu_Mod_L
   fi
}

Menu_Mod_V ()
{
   MESSAGE="Verifying all ServerPackages ..."
   ERR=0

   for RETVAL in `cat $SYS_DIR/*.int | sed -n -e "s/Object=(\(.*\))/\\1/" -e "t label" -e "d" -e ": label" -e "p" | gawk '
BEGIN {
   FS=",";
   mem="";
}
function xcv(input) {
   if (substr(input, 0, 5) == "Name=") {
      tmp=substr(input, 6);
      match(tmp, "[.]");
      if (RSTART > 0) {
         name=substr(tmp, 0, RSTART - 1);
      } else {
         name="";
      }
   }
   if (input == "MetaClass=Engine.Mutator") {
      mutator=1;
   }
}
{
   mutator=0;
   for (i = 1; i <= NF; i ++) {
      match (\$i, "\""); if (RSTART > 0) {
         if (length(mem) == 0) {
            mem = \$i;
         } else {
            mem = mem "," \$i; xcv(mem); mem = "";
         }
      } else {
         if (length(mem) == 0) {
            xcv(\$i);
         } else {
            mem = mem "," \$i;
         }
      }
   }
   if (mutator > 0) {
      print name;
   }
}'`; do
      MY_U="$SYS_DIR/$RETVAL"".u"
      if [ ! -f "$MY_U" ]; then
         Add_Message "Can not find the file $MY_U required by one or more .int file(s)"
         ERR=1
      fi
   done

   for INFILE in `cat $UT_MAIN_INI | sed -n -e "s/^ServerPackages=\([a-zA-Z0-9]*\).*$/\\1/" -e "t label" -e "d" -e ": label" -e "p"`; do
      MY_INT="$INFILE"".int"
      if [ ! -f "$SYS_DIR/$MY_INT" ]; then
         Add_Message "Can not find required ServerPackage: $MY_INT"
         ERR=1
      fi
   done

   for INFILE in `ls $SYS_DIR/*.u | sed -e "s/^.*\/\([^\/]*\)\.u$/\\1/"`; do
      TMP=`grep -i "packages=$INFILE" $UT_MAIN_INI`
      if [ -z "$TMP" ]; then
         Add_Message "$INFILE.u is not being initialized by the .ini file."
         ERR=1
      fi
   done

   if [ $ERR -eq 0 ]; then
      Add_Message "Everything looks fine."
   else
      Add_Message "At least one error was found but this version of asu can not fix it yet."
   fi
}

Initialization

# EOF
